/*
    \brief  Deklarationen der Funktionen für die Arbeit mit den Setuplisten.

    \author Bernd Ottmann
    \date   21.02.2003
*/
#pragma once
#ifndef DATAFOX_SETUP_LISTS_H
#ifndef DOXYGEN_SHOULD_SKIP_THIS
#define DATAFOX_SETUP_LISTS_H
#endif

#include "df_defines.h"

/*!
    \brief Bibliotheksinterne Datenpuffer für Setuplisten löschen und initalisieren.

    Die in der Bibliothek bereitgestellten Datenpuffer für die zu übertragenenden Listendaten werden gelöscht und
    für eine Verwendung mittels DFCMakeListe() und DFCLoadListen() initalisiert.

    \ingroup setup_lists

    \export{_DFCClrListenBuffer@4, DFCClrListenBuffer}

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
*/
DF_API_EXTERN_DECORATE void DF_API_CALL DFCClrListenBuffer( int connectionNumber );

/*!
    \brief Kopieren eines aufbereiteten Datenarrays für eine Setupliste in den Biblioteksinternen Datenpuffer.

    Das zu übergebende Datenarray wird in den Bibliotheksinternen Datenpuffer kopiert und für eine spätere
    Übertragung mittels DFCLoadListen() zwischengespeichert.

    \ingroup setup_lists

    \export{_DFCMakeListe@24, DFCMakeListe}

    \note Für die Verarbeitung der Listendaten siehe [Listen des Setups](\ref dfc_lists).

    \note Für die Aufbereitung der Listendaten siehe [Importieren von Listendaten](\ref dfc_lists_import).

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] number Listennnummer.
    \param[in] recordCount Anzahl der Datensätze im übergebenen Datenarray.
    \param[in] size Anzahl Byte des Datenarrays.\n Brechnet sich aus (recordCount * Größe pro Datensatz).
    \param[in] buffer Datenarray mit den Daten.
    \param[in] unused_setTo0 Sollte mit 0 übergeben werden. Reservierter Parameter.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCMakeListe( int connectionNumber, int number, int recordCount, int size, const unsigned char *buffer, int unused_setTo0 );

/*!
    \brief Übertragen der bibliotheksinternen Datenpuffer zum Gerät.

    Es werden die mit DFCMakeListe() importierten Listendaten in das Gerät geschrieben. Vorhandene Listendaten
    werden hierbei erst nach einer erfolgreichen Übernahme überschrieben.

    \ingroup setup_lists

    \export{_DFCLoadListen@12, DFCLoadListen}

    \note Für die Verarbeitung der Listendaten siehe [Listen des Setups](\ref dfc_lists).

    \attention Bitte beachten Sie die [Hinweise zur Lebensdauer](\ref dfc_lists_memory) der eingesetzten Flash-Bausteine.
    Diese gelten ebenfalls für Zutrittslisten.

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCLoadListen( int connectionNumber, int deviceAddress, int *errorNumber );

/*!
    \brief Aufbau der Listentabellen aus dem Geräte lesen.

    Es werden die im Gerätesetup verwendeten Tabellenbeschreibungen für die Setuplisten in den Bibliotheksinternen
    Puffer gelesen. Hiernach kann mit den Funktionen DFCListBCnt(), DFCListBDatensatz() und DFCListBFeld()
    auf die ermittelten Daten zugegriffen werden.

    \ingroup setup_lists

    \export{_DFCLoadListenbeschreibung@12, DFCLoadListenbeschreibung}

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] deviceAddress Anzusprechende Geräteadresse.
    \param[out] errorNumber Fehlernummer bei fehlerhafter Ausführung.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCLoadListenbeschreibung( int connectionNumber, int deviceAddress, int *errorNumber );

/*!
    \brief Anzahl der gelesenen Listentabellen abrufen.

    Auf Grundlage der zuvor mit DFCLoadListenbeschreibung() gelesenen Listentabellen, wird
    die im Gerätesetup definierte Anzahl zurückgegeben.

    \ingroup setup_lists

    \export{_DFCListBCnt@4, DFCListBCnt}

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.

    \return Anzahl der Listentabellen.
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCListBCnt( int connectionNumber );

/*!
    \brief Grunddaten einer Listentabelle abrufen.

    Auf Grundlage der zuvor mit DFCLoadListenbeschreibung() gelesenen Listentabellen, werden
    die Grunddaten zu einer Listentabelle zurückgegeben.

    \ingroup setup_lists

    \export{_DFCListBDatensatz@20, DFCListBDatensatz}

    \note Für das Arbeiten mit den Strukturen der Zutrittslisten ist kein vorheriges Laden mit DFCLoadListenbeschreibung() nötig.

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] number Listennummer. 
    <table class="paramvalues">
        <tr><th>Bereich</th><th>Bedeutung</th></tr>
        <tr><td>0 bis DFCListBCnt() - 1</td><td>Nummer der Listenbeschreibung aus dem Setup</td></tr>
        <tr><td>100 bis 103</td><td>Ab DFCom Version 02.00.22 für die Zutrittslisten Version 1 (z. B. 102 für Liste "Personalstamm")<br />Offsetnummern der Listen siehe DFCMakeEntranceList()</td></tr>
        <tr><td>200 bis 207</td><td>Ab DFCom Version 04.01.00 für die Zutrittslisten Version 2 (z. B. 203 für Liste "Time")<br />Offsetnummern der Listen siehe DFCMakeEntrance2List()</td></tr>
    </table>  
    \param[out] name Zeichenpuffer für den Namen. Muss mindestens 17 Zeichen lang sein.
    \param[out] fieldCount Anzahl der Felder.
    \param[out] unused_setTo0 Variable welche eine 0 erhält. Reservierter Parameter.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCListBDatensatz( int connectionNumber, int number, char *name, int *fieldCount, int *unused_setTo0 );

/*!
    \brief Grunddaten eines Listenfeldes abrufen.

    Auf Grundlage der zuvor mit DFCLoadListenbeschreibung() gelesenen Listentabellen, werden
    die Grunddaten zu einem Listenfeld zurückgegeben.
    
    \ingroup setup_lists

    \export{_DFCListBFeld@24, DFCListBFeld}

    \note Für das Arbeiten mit den Strukturen der Zutrittslisten ist kein vorheriges Laden mit DFCLoadListenbeschreibung() nötig.

    \param[in] connectionNumber Zu verwendende Verbindungsnummer.
    \param[in] listBNumber Listennummer.
        <table class="paramvalues">
        <tr><th>Bereich</th><th>Bedeutung</th></tr>
        <tr><td>0 bis DFCListBCnt() - 1</td><td>Nummer der Listenbeschreibung aus dem Setup</td></tr>
        <tr><td>100 bis 103</td><td>Ab DFCom Version 02.00.22 für die Zutrittslisten Version 1 (z. B. 102 für Liste "Personalstamm")<br />Offsetnummern der Listen siehe DFCMakeEntranceList()</td></tr>
        <tr><td>200 bis 207</td><td>Ab DFCom Version 04.01.00 für die Zutrittslisten Version 2 (z. B. 203 für Liste "Time")<br />Offsetnummern der Listen siehe DFCMakeEntrance2List()</td></tr>
    </table>
    \param[in] number Feldnummer. Bereich 0 bis maximal 25.
    \param[out] name Zeichenpuffer für den Feldnamen. Muss mindestens 17 Zeichen lang sein.
    \param[out] type Datentyp.
    <table class="paramvalues">
        <tr><th>Wert</th><th>Bedeutung</th></tr>
        <tr><td>1</td><td>uint32 (4 Byte)</td></tr>
        <tr><td>2</td><td>\ref dfc_type_date_time "DateTime" (7 Byte, 4 Byte Date, 3 Byte Time)</td></tr>
        <tr><td>3</td><td>String (1-41 Byte, only digits [0-9])</td></tr>
        <tr><td>4</td><td>String (1-41 Byte, Latin1 codec)</td></tr>
        <tr><td>5</td><td>Date (4 Byte; Y100, Y, M, D)</td></tr>
        <tr><td>6</td><td>Time (3 Byte; h, m, s)</td></tr>
        <tr><td>7</td><td>Finger template (161 Byte; DIN V66400)</td></tr>
        <tr><td>8</td><td>Finger template (216 Byte; Idencom Compact)</td></tr>
        <tr><td>9</td><td>Binar data (1-220 Byte, Binär)</td></tr>
        <tr><td>10</td><td>\ref dfc_type_data "Daten" (2-241 Byte, Binär)</td></tr>
    </table>
    \param[out] size Länge. Die Längenangabe beinhaltet ein eventuell benötigtes Nullterminierungszeichen bei Feldern
    für Zeichenfolgen.

    \retval 0 Fehler bei Ausführung.
    \retval 1 Ausführung war erfolgreich.
*/
DF_API_EXTERN_DECORATE int DF_API_CALL DFCListBFeld( int connectionNumber, int listBNumber, int number, char *name, int *type, int *size );

#endif
