/****************************************************************************
**
** Copyright (C) Datafox GmbH.
** All rights reserved.
** Contact: Datafox GmbH (develop@datafox.de)
**
** This file provides a collection of helper functions.
**
****************************************************************************/
#include "common/utils.h"
#include <string>
#include <fstream>
#include <memory>

#if defined(_MSC_VER) || defined(__MINGW32__)
#ifndef WIN32_LEAN_AND_MEAN
#define WIN32_LEAN_AND_MEAN
#endif
#include <windows.h>
#endif

#if !defined(_MSC_VER)
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#endif

#include <boost/algorithm/string.hpp>
#include <boost/uuid/uuid.hpp>
#include <boost/uuid/uuid_generators.hpp>
#include <boost/uuid/uuid_io.hpp>
#include <boost/format.hpp>


namespace Utils
{
    namespace File
    {
        bool exists( const std::string &name )
        {
            struct stat buffer;
            return (stat( name.c_str(), &buffer ) == 0);
        }

        const std::string extension( const std::string &filePath )
        {
            std::string extension;

            // Find the last dot, if any.
            size_t dotIdx = filePath.find_last_of( "." );
            if ( dotIdx != std::string::npos )
            {
                // Find the last directory separator, if any.
                size_t dirSepIdx = filePath.find_last_of( "/\\" );

                // If the dot is at the beginning of the file name, do not treat it as a file extension.
                // e.g., a hidden file:  ".alpha".
                // This test also incidentally avoids a dot that is really a current directory indicator.
                // e.g.:  "alpha/./bravo"
                if ( dotIdx > dirSepIdx + 1 )
                {
                    extension = filePath.substr( dotIdx + 1 );
                }
            }
            return extension;
        }

        const std::string name( const std::string &filePath )
        {
            std::string name;

            std::function<size_t( size_t )> sizeOfFilename = [&]( size_t startIdx )
            {
                size_t count{ std::string::npos };
                // Find the last dot, if any.
                size_t dotIdx = filePath.find_last_of( "." );
                if ( dotIdx != std::string::npos )
                {
                    if ( startIdx == 0 )
                    {
                        count = dotIdx;
                    }
                    else if ( dotIdx > startIdx )
                    {
                        count = dotIdx - startIdx;
                    }
                }
                else
                {
                    size_t filePathSize = filePath.size();
                    if ( filePathSize > startIdx )
                    {
                        count = filePathSize - startIdx;
                    }
                }
                return count;
            };

            // Find the last directory separator, if any.
            size_t dirSepIdx = filePath.find_last_of( "/\\" );
            if ( dirSepIdx != std::string::npos )
            {
                name = filePath.substr( dirSepIdx + 1, sizeOfFilename( dirSepIdx + 1 ) );
            }
            else
            {
                name = filePath.substr( 0, sizeOfFilename( 0 ) );
            }
            return name;
        }

        const std::string dir( const std::string &filePath )
        {
            if ( File::extension( filePath ).empty() )
            {
                return filePath;
            }

            std::string dir;

            // Find the last directory separator, if any.
            size_t dirSepIdx = filePath.find_last_of( "/\\" );
            if ( dirSepIdx != std::string::npos )
            {
                dir = filePath.substr( 0, dirSepIdx + 1 );
            }
            return dir;
        }

        bool copy( const std::string &srcPath, const std::string &dstPath )
        {
            bool bResult = false;
            std::ifstream src( srcPath, std::ios::binary );
            std::ofstream dst( dstPath, std::ios::binary );

            if ( src.good() && dst.good() )
            {
                dst << src.rdbuf();
                bResult = true;
            }
            src.close();
            dst.close();
            return bResult;
        }

        const std::string &replaceDirSeparator( std::string &path )
        {
            boost::replace_all( path, "\\", "/" );
            return path;
        }

        const std::string uniqueFileName( const std::string &prefix, const std::string &extension )
        {
            std::string fileName = prefix + boost::replace_all_copy( to_string( boost::uuids::random_generator()() ), "-", "" );
            if ( !extension.empty() )
            {
                if ( extension[0] != '.' )
                {
                    fileName += ".";
                }

                fileName += extension;
            }

            return fileName;
        }

        const std::string uniqueTempFileName( const std::string &prefix, const std::string &extension )
        {
            std::string tempFileName( tempDirectoryPath() + "/" + uniqueFileName( prefix, extension ) );
            // Prüfen ob die Datei dort existiert.
            struct stat lstat;
            if ( stat( tempFileName.c_str(), &lstat ) == -1 || (lstat.st_mode & S_IFMT) != S_IFREG )
            {
                // Nur zur Sicherheit, sollte nicht vorkommen, dass eine solche Datei bereits existiert.
                tempFileName = tempDirectoryPath() + "/" + uniqueFileName( prefix, extension );
            }

            return tempFileName;
        }

    } // namespace File

    const std::string applicationDirectoryPath()
    {
        std::string executableFilePath( 260, '\0' );

#if defined(_MSC_VER) || defined(__MINGW32__)
        // Datei der Anwendung
        if ( GetModuleFileNameA( nullptr, &executableFilePath[0], (DWORD)executableFilePath.size() ) == 0 )
        {
            // Fehler bei Ermittlung
            executableFilePath.clear();
        }
        else
        {
            // Windows akzeptiert auch '/' als Pfadtrenner, somit Gemeinsamkeit mit GNU/Linux herstellen.
            boost::replace_all( executableFilePath, "\\", "/" );
        }
#else
        // Datei der Anwendung
        std::string tmp( 260, '\0' );
        std::sprintf( &tmp[0], "/proc/%d/exe", getpid() );
        if ( readlink( tmp.c_str(), &executableFilePath[0], executableFilePath.size() ) == -1 )
        {
            // Fehler bei Ermittlung
            executableFilePath.clear();
        }
#endif
        // Eventuelle Nullterminierungen entfernen
        auto firstPos = executableFilePath.find_first_of( '\0' );
        if ( firstPos != std::string::npos )
        {
            executableFilePath.resize( firstPos );
        }

        return File::dir( executableFilePath );
    }

    const std::string tempDirectoryPath()
    {
        std::string result;
#if defined( _MSC_VER ) || defined( __MINGW32__ )
    // Unter Windows
    // TMP, TEMP, USERPROFILE
    // %windir%

        // Benötigte Puffergröße ermitteln.
        DWORD bufferSize = GetTempPathA( 0, nullptr );
        if ( bufferSize > 3 ) // Mindestens Laufwerksbuchstabe mit Abschluß
        {
            // TODO: Noch nicht Unicodetauglich
            result.resize( bufferSize );
            GetTempPathA( static_cast<DWORD>( result.size() ), &result[0] );
            // Windows akzeptiert auch '/' als Pfadtrenner, somit Gemeinsamkeit mit GNU/Linux herstellen.
            boost::replace_all( result, "\\", "/" );
            if ( '/' == result[result.size() - 2] )
            {
                result[result.size() - 2] = 0;
            }
        }
#else
    // Unter Linux ISO/IEC 9945 (POSIX)
    // TMPDIR, TMP, TEMP, TEMPDIR
    // /tmp (bei __ANDROID__) /data/local/tmp  
        std::vector<std::string> envVars{ "TMPDIR", "TMP", "TEMP", "TEMPDIR" };

        // Versuchen eine gültige Umgebungsvariable für den Temp-Pfad zu finden.
        std::string env_temp_path;
        for ( std::size_t i = 0; i < envVars.size(); ++i )
        {
            char *temp_path = std::getenv( envVars[i].c_str() );
            if ( nullptr != temp_path )
            {
                env_temp_path = temp_path;

#if defined(_MSC_VER) || defined(__MINGW32__)
                // Windows akzeptiert auch '/' als Pfadtrenner, somit Gemeinsamkeit mit GNU/Linux herstellen.
                boost::replace_all( env_file_path, "\\", "/" );
#endif

                if ( env_temp_path.size() )
                {
                    if ( '/' == env_temp_path[env_temp_path.size() - 1] )
                    {
                        env_temp_path[env_temp_path.size() - 1] = 0;
                    }

                    // Nur, wenn das Verzeichnis existiert ist dieses gültig.
                    struct stat lstat;
                    if ( stat( env_temp_path.c_str(), &lstat ) == 0 && (lstat.st_mode & S_IFMT) == S_IFDIR )
                    {
                        // Existierendes Verzeichnis als Temp-Verzeichnis verwenden.
                        result = env_temp_path;
                        break;
                    }
                }
            }
        }

        if ( result.empty() )
        {
            // Standardpfad für Linux vorgeben.
            result = "/tmp";
        }
#endif
        // Eventuelle Nullterminierungen entfernen
        auto firstPos = result.find_first_of( '\0' );
        if ( firstPos != std::string::npos )
        {
            result.resize( firstPos );
        }

        return result;
    }

    const std::string uniqueTempDirectoryPath( const std::string &prefix )
    {
        std::string tempPath( tempDirectoryPath() + "/" + prefix + boost::replace_all_copy( to_string( boost::uuids::random_generator()() ), "-", "" ) );
        // Nur, wenn das Verzeichnis existiert ist dieses gültig.
        struct stat lstat;
        if ( stat( tempPath.c_str(), &lstat ) == 0 && (lstat.st_mode & S_IFMT) == S_IFDIR )
        {
            // Nur zur Sicherheit, sollte nicht vorkommen, dass ein solches Verzeichnis bereits existiert.
            tempPath = tempDirectoryPath() + "/" + prefix + boost::replace_all_copy( to_string( boost::uuids::random_generator()() ), "-", "" );
        }

        return tempPath;
    }

    std::string toHexString( const uint8_t *data, uint32_t length )
    {
        if ( data != nullptr )
        {
            std::string s( length * 2, ' ' );
            for ( uint32_t i = 0; i < length; ++i )
            {
                s[2 * i] = hexmap[(data[i] & 0xF0) >> 4];
                s[2 * i + 1] = hexmap[data[i] & 0x0F];
            }
            return s;
        }
        return "";
    }

    std::string toHexString( const uint32_t& data, bool withPrefix )
    {
        constexpr uint8_t size = sizeof( data );
        uint8_t prefixCount = (withPrefix ? 2 : 0);
        std::string s( size * 2 + prefixCount, ' ' );
        
        // add prefix to result string
        if ( withPrefix )
        {
            s[0] = '0';
            s[1] = 'x';
        }

        uint8_t tmp{};
        for ( uint32_t i = 0; i < size; ++i )
        {
            tmp = data >> ((size - i - 1) * 8);
            s[2 * i + prefixCount] = hexmap[(tmp & 0xF0) >> 4];
            s[2 * i + 1 + prefixCount] = hexmap[tmp & 0x0F];
        }
        return s;
    }

    std::vector<uint8_t> fromHexString(const std::string& data, bool swapOrder )
    {
        std::vector<uint8_t> result{};

        // The string must consist of an even number of characters.
        if ( (data.size() % 2) == 0 )
        {
            uint16_t startPos = 0;
            // detect prefix and remove it (further processing starts after the prefix)
            if ( data.find( "0x" ) == 0 )
            {
                startPos = 2;
            }

            for ( int i = startPos; i < static_cast<int>(data.size()); i += 2 )
            {
                result.push_back( static_cast<uint8_t>(std::stoul( data.substr( i, 2 ), nullptr, 16 )) );
            }

            // Swap the order in the vector, because the interpretation of a number stored as a string runs in reverse order.
            if ( swapOrder )
            {
                std::reverse( result.begin(), result.end() );
            }
        }
        return result;
    }

    const std::string replaceAll( const std::string &value, const std::string &search, const std::string &replace )
    {
        std::string newString = value;
        boost::replace_all( newString, search, replace );
        return newString;
    }

    std::string unicodeToUTF8( const std::wstring &wstr )
    {
        std::string ret;
        try
        {
            std::wstring_convert< std::codecvt_utf8<wchar_t> > wcv;
            ret = wcv.to_bytes( wstr );
        }
        catch ( const std::exception & e )
        {
            std::cerr << e.what() << std::endl;
        }
        return ret;
    }
    std::wstring utf8ToUnicode( const std::string &str )
    {
        std::wstring ret;
        try
        {
            std::wstring_convert< std::codecvt_utf8<wchar_t> > wcv;
            ret = wcv.from_bytes( str );
        }
        catch ( const std::exception & e )
        {
            std::cerr << e.what() << std::endl;
        }
        return ret;
    }

    std::string unicodeToANSI( const std::wstring &wstr )
    {
        std::string ret;
        std::mbstate_t state = {};
        const wchar_t *src = wstr.data();
        size_t len = std::wcsrtombs( nullptr, &src, 0, &state );
        if ( static_cast<size_t>(-1) != len )
        {
            std::unique_ptr< char[] > buff( new char[len + 1] );
            len = std::wcsrtombs( buff.get(), &src, len, &state );
            if ( static_cast<size_t>(-1) != len )
            {
                ret.assign( buff.get(), len );
            }
        }
        return ret;
    }
    std::wstring ansiToUnicode( const std::string &str )
    {
        std::wstring ret;
        std::mbstate_t state = {};
        const char *src = str.data();
        size_t len = std::mbsrtowcs( nullptr, &src, 0, &state );
        if ( static_cast<size_t>(-1) != len )
        {
            std::unique_ptr< wchar_t[] > buff( new wchar_t[len + 1] );
            len = std::mbsrtowcs( buff.get(), &src, len, &state );
            if ( static_cast<size_t>(-1) != len )
            {
                ret.assign( buff.get(), len );
            }
        }
        return ret;
    }

    std::string to_utf8( const std::string &str )
    {
        return unicodeToUTF8( ansiToUnicode( str ) );
    }
    std::string from_utf8( const std::string &str )
    {
        return unicodeToANSI( utf8ToUnicode( str ) );
    }
    std::string from_utf8( const char *str )
    {
        return (str != nullptr) ? from_utf8( std::string( str ) ) : "";
    }

    namespace Debug
    {
        void printToConsole( const char *format, ... )
        {
#if defined(_MSC_VER)
            va_list argptr;
            char buffer[512];

            va_start( argptr, format );
            vsnprintf( buffer, sizeof( buffer ), format, argptr );
            va_end( argptr );

            OutputDebugStringA( buffer );
#else
            (void)format;
#endif
        }
    }
}
